@@ -506,6 +506,8 @@ export class Application<T extends Widget = Widget> {
     // Mark the application as started;
     this._started = true;
 
+    this._bubblingKeydown = options.bubblingKeydown || false;
+
     // Parse the host ID for attaching the shell.
     const hostID = options.hostID || '';
 
@@ -607,7 +609,7 @@ export class Application<T extends Widget = Widget> {
    */
   protected addEventListeners(): void {
     document.addEventListener('contextmenu', this);
-    document.addEventListener('keydown', this, true);
+    document.addEventListener('keydown', this, !this._bubblingKeydown);
     window.addEventListener('resize', this);
   }
 
@@ -663,6 +665,7 @@ export class Application<T extends Widget = Widget> {
   private _plugins = new Map<string, Private.IPluginData>();
   private _services = new Map<Token<any>, string>();
   private _started = false;
+  private _bubblingKeydown = false;
 }
 
 /**
@@ -715,6 +718,14 @@ export namespace Application {
      * This will override `startPlugins` and any `autoStart` plugins.
      */
     ignorePlugins?: string[];
+
+    /**
+     * Whether to capture keydown event at bubbling or capturing (default) phase for
+     * keyboard shortcuts.
+     *
+     * @experimental
+     */
+    bubblingKeydown?: boolean;
   }
 }
 
