@@ -1615,6 +1615,11 @@ describe('@lumino/widgets', () => {
           expect(bar.addButtonNode.getAttribute('tabindex')).to.equal('-1');
         });
 
+        it('should have a role attribute of button', () => {
+          populateBar(bar);
+          expect(bar.addButtonNode.getAttribute('role')).to.equal('button');
+        });
+
         it('should focus the second tab on right arrow keydown', () => {
           populateBar(bar);
           const firstTab = bar.contentNode.firstChild as HTMLElement;
